from Products.CMFCore.utils import getToolByName
from ftw.book.browser.indexer import book_keywords
from ftw.book.tests import FunctionalTestCase
from ftw.builder import Builder
from ftw.builder import create
from plone.app.textfield.value import RichTextValue
from plone.mocktestcase.dummy import Dummy
from unittest import TestCase


class TestUnitKeywords(TestCase):

    def test_extracts_keywords_from_text(self):
        html = '<p>Foo <span class="keyword" title="bar">Bar</span> Baz</p>'
        obj = Dummy(text=RichTextValue(html))
        self.assertEquals(['bar'], book_keywords(obj)())

    def test_extracts_multiple_keywords(self):
        html = '\n'.join((
                '<p>',
                ' <span class="keyword" title="Foo">Foo</span>',
                ' <span class="keyword" title="barX">Bar</span>',
                ' <span class="keyword" title="Baz">Baz</span>',
                '</p>'))
        obj = Dummy(text=RichTextValue(html))

        self.assertEquals(['Foo', 'barX', 'Baz'],
                          book_keywords(obj)())

    def test_returns_empty_list_when_no_text_is_set(self):
        obj = Dummy(text=None)
        self.assertEquals([],
                          book_keywords(obj)())

    def test_does_not_include_empty_keyword_nodes(self):
        html = '\n'.join((
                '<p>',
                ' <span class="keyword" title="Foo">Foo</span>',
                ' <span class="keyword" title="">Bar</span>',
                ' <span class="keyword">Baz</span>',
                '</p>'))
        obj = Dummy(text=RichTextValue(html))
        self.assertEquals(['Foo'], book_keywords(obj)())

    def test_strips_whitespace(self):
        html = '\n'.join((
                '<p>',
                ' <span class="keyword" title=" Foo Bar ">Foo Bar</span>',
                '</p>'))
        obj = Dummy(text=RichTextValue(html))
        self.assertEquals(['Foo Bar'], book_keywords(obj)())

    def test_works_with_multiple_root_nodes(self):
        html = '\n'.join((
                '<span class="keyword" title="Foo">Foo</span>',
                '<span class="keyword" title="Bar">Bar</span>'))
        obj = Dummy(text=RichTextValue(html))
        self.assertEquals(['Foo', 'Bar'], book_keywords(obj)())

    def test_unicode_umlauts(self):
        html = u'\n'.join((
                u'<p>',
                u' <span class="keyword" title="hall\xf6chen">Hi</span>',
                u'</p>'))
        obj = Dummy(text=RichTextValue(html))
        self.assertEquals([u'hall\xf6chen'.encode('utf-8')],
                          book_keywords(obj)())

    def test_utf8_umlauts(self):
        html = u'\n'.join((
                u'<p>',
                u' <span class="keyword" title="hall\xf6chen">Hi</span>',
                u'</p>')).encode('utf-8')
        obj = Dummy(text=RichTextValue(html))
        self.assertEquals([u'hall\xf6chen'.encode('utf-8')],
                          book_keywords(obj)())


class TestKeywordIndex(FunctionalTestCase):

    def test_indexes_blocks(self):
        self.grant('Manager')
        html = '<p><span class="keyword" title="Foo">Foo</span></p>'
        block = create(Builder('book textblock')
                       .with_text(html)
                       .within(self.example_book.empty))

        catalog = getToolByName(self.layer['portal'], 'portal_catalog')
        rid = catalog.getrid(path='/'.join(block.getPhysicalPath()))
        index_data = catalog.getIndexDataForRID(rid)

        self.assertDictContainsSubset(
            {'book_keywords': ['Foo']},
            index_data)

    def test_keyword_metadata(self):
        html = '<p><span class="keyword" title="Foo">Foo</span></p>'
        block = create(Builder('book textblock')
                       .with_text(html)
                       .within(self.example_book.empty))

        catalog = getToolByName(self.layer['portal'], 'portal_catalog')
        brain = catalog({'path': '/'.join(block.getPhysicalPath())})[0]
        self.assertEquals(['Foo'],
                          brain.book_keywords)



from ftw.book import IS_PLONE_5
from ftw.upgrade import UpgradeStep


class UpdateResourceBundle(UpgradeStep):
    """Update resource bundle.
    """

    def __call__(self):
        # The registry entries only have to be installed on plone 5
        if IS_PLONE_5:
            self.install_upgrade_profile()



from ftw.book import _
from ftw.book.interfaces import IChapter
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.content import Container
from plone.supermodel.model import Schema
from zope.interface import implements
from zope.interface import provider
from zope.schema import TextLine


@provider(IFormFieldProvider)
class IChapterSchema(Schema):

    title = TextLine(
        title=_(u'label_title', default=u'Title'),
        required=True,
    )


class Chapter(Container):
    implements(IChapter)

    # Prevent acquisition in exclude_from_nav indexer
    # since when IExcludeFromNavigation is not enabled.
    exclude_from_nav = False



from zope.i18nmessageid import MessageFactory
import pkg_resources


_ = MessageFactory('ftw.book')
IS_PLONE_5 = pkg_resources.get_distribution('Products.CMFPlone').version >= '5'


def initialize(context):
    pass



from ftw.upgrade import UpgradeStep


class FixNavigationAndSearchForNewTypes(UpgradeStep):
    """Fix navigation and search for new types.
    """

    def __call__(self):
        self.install_upgrade_profile()



from ftw.book.table.generator import TableGenerator
from ftw.book.tests import FunctionalTestCase
from ftw.book.tests.builders import asset
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import Browser
from ftw.testbrowser import browsing
from ftw.testbrowser.exceptions import HTTPServerError
from plone.app.textfield.value import RichTextValue
import textwrap


class TestTableExportImport(FunctionalTestCase):

    def setUp(self):
        super(TestTableExportImport, self).setUp()
        self.table = self.example_book.unrestrictedTraverse(
            'historical-background/china/population')

    @browsing
    def test_export_table(self, browser):
        browser.login().visit(self.table, view='table_export_import')
        browser.click_on('Export')

        self.assertEqual(textwrap.dedent('''
        http://nohost/plone/the-example-book/historical-background/china/population;;
        Ranking;City;Population
        1;Guangzhou;44 mil <sup>1</sup>
        2;Shanghai;35 mil
        3;Chongqing;30 mil
        ''').replace('\n', '\r\n').strip(), browser.contents.strip())

        self.assertEqual(
            'text/csv; charset=utf-8',
            browser.headers.get('content-type'))

    @browsing
    def test_non_ascii_export(self, browser):
        table2 = create(Builder('table')
                        .titled(u'Population')
                        .with_table((('Ranking', 'City', 'Population'),
                                     ('0', u'Gu\xe4ngzhou', '44 mil <sup>1</sup>')))
                        .having(border_layout='grid',
                                footnote_text=RichTextValue(
                                    u'<p><sup>1</sup> thats quite big</p>'))
                        .within(self.table.aq_parent))
        browser.login().visit(table2, view='table_export_import')
        try:
            browser.click_on('Export')
        except HTTPServerError:
            self.fail('Export should work non ascii chars in the table')

    @browsing
    def test_bom_encoded_import(self, browser):
        self.assert_table_display(
            [['Ranking', 'City', 'Population'],
             ['1', 'Guangzhou', '44 mil 1'],
             ['2', 'Shanghai', '35 mil'],
             ['3', 'Chongqing', '30 mil']])

        csvfile = (u'http://nohost/plone/the-example-book/historical-background/china/population;;\n\r'
                   u'Ranking;City;Population\r\n'
                   u'1;Gu\xe4ngzhou;100 mil').encode('utf-8-sig')  # Encode as utf-8 with BOM

        browser.login().visit(self.table, view='table_export_import')
        browser.fill({'Modified CSV-file:': (csvfile, 'file.csv', 'text/csv'),
                      'Columns to import': 'City (Spalte 2)'})

        browser.click_on('Import')

        self.assert_table_display(
            [['Ranking', 'City', 'Population'],
             ['1', u'Gu\xc3\xa4ngzhou', '44 mil 1'],
             ['2', 'Shanghai', '35 mil'],
             ['3', 'Chongqing', '30 mil']])

    @browsing
    def test_import_table(self, browser):
        self.assert_table_display(
            [['Ranking', 'City', 'Population'],
             ['1', 'Guangzhou', '44 mil 1'],
             ['2', 'Shanghai', '35 mil'],
             ['3', 'Chongqing', '30 mil']])

        csvfile = textwrap.dedent('''
        http://nohost/plone/the-example-book/historical-background/china/population;;
        Ranking;City;Population!!!!!!!
        1;Guangzhou;100 mil
        2;Shanghai;101 mil
        3;Chongqing;102 mil
        ''').replace('\n', '\r\n').strip()
        browser.login().visit(self.table, view='table_export_import')
        browser.fill({'Modified CSV-file:': (csvfile, 'file.csv', 'text/csv'),
                      'Columns to import': 'Population (Spalte 3)'})
        browser.click_on('Import')

        self.assert_table_display(
            [['Ranking', 'City', 'Population'],
             ['1', 'Guangzhou', '100 mil'],
             ['2', 'Shanghai', '101 mil'],
             ['3', 'Chongqing', '102 mil']])

    def assert_table_display(self, expected):
        with Browser() as browser:
            browser.open_html(TableGenerator(self.table).render())
            got = browser.css('table').first.lists()

        self.assertEqual(expected, got)



from ftw.book.behaviors.codeinjection import ILaTeXCodeInjection
from ftw.upgrade import UpgradeStep
from Products.CMFPlone.utils import safe_unicode


class FixBaseUnitInILaTeXCodeInjectionBehavior(UpgradeStep):
    """Fix BaseUnit in ILaTeXCodeInjection behavior.
    """

    def __call__(self):
        self.install_upgrade_profile()

        query = {'object_provides': [ILaTeXCodeInjection.__identifier__]}
        for obj in self.objects(query, 'Fix pre post latex code field'):
            if callable(ILaTeXCodeInjection(obj).pre_latex_code):
                ILaTeXCodeInjection(obj).pre_latex_code = safe_unicode(
                    ILaTeXCodeInjection(obj).pre_latex_code()
                )
            if callable(ILaTeXCodeInjection(obj).post_latex_code):
                ILaTeXCodeInjection(obj).post_latex_code = safe_unicode(
                    ILaTeXCodeInjection(obj).post_latex_code()
                )
